<?php

namespace Drupal\commerce_adyen_cc\Plugin\Commerce\PaymentGateway;

use Drupal\commerce_payment\Entity\PaymentInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\OnsitePaymentGatewayInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsAuthorizationsInterface;
use Drupal\commerce_payment\Plugin\Commerce\PaymentGateway\SupportsRefundsInterface;
use Drupal\profile\Entity\ProfileInterface;

/**
 * Provides the interface for the Web Component payment gateway.
 */
interface WebComponentInterface extends OnsitePaymentGatewayInterface, SupportsAuthorizationsInterface, SupportsRefundsInterface {

  /**
   * Get the session data.
   *
   * @return array
   *   The session data.
   */
  public function createPaymentSession(): array;

  /**
   * Creates a payment method with the given payment details.
   *
   * @param array $payment_details
   *   The gateway-specific payment details provided by the payment method form
   *   for on-site gateways, or the incoming request for off-site gateways.
   * @param \Drupal\profile\Entity\ProfileInterface $billing_profile
   *   The billing profile.
   */
  public function authenticate(array $payment_details, ProfileInterface $billing_profile): void;

  /**
   * Perform $0 authorization when 3ds2 is NOT required.
   *
   * Used for tokenization.
   *
   * @param array $data
   *   The data.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function zeroDollarAuthorize3ds2NotRequired(array $data): void;

  /**
   * Perform authorization.
   *
   * @param \Drupal\commerce_payment\Entity\PaymentInterface $payment
   *   The payment.
   *
   * @throws \Drupal\Core\Entity\EntityStorageException
   */
  public function authorize(PaymentInterface $payment): void;

  /**
   * Handle a 3ds2 action.
   *
   * @param array $data
   *   The data.
   */
  public function zeroDollarAuthorize3ds2Required(array $data): void;

  /**
   * Return an array of allowed country codes.
   *
   * @return array
   *   The allowed country codes.
   */
  public function getBillingAddressAllowedCountries(): array;

}
